/*  GnuPOSkeyboard.c                      */
/*  This contains the keyboard interface  */
/*  routines for GnuPOS, including the    */
/*  reading and writing of the xml state  */
/*  definitions file.                     */
/*                                        */
/*  (c) 2001 Conrad Canterford.           */

#include <gnome.h>
#include <gnome-xml/xmlmemory.h>
#include <gnome-xml/parser.h>

#include "GnuPOS.h"
#include "GnuPOS-P.h"
#include "GnuPOS-keyboard-P.h"
#include "GnuPOS-keymap-defines.h"

state_master_struct all_states;

/*                                         */
/*       KEYMAP ROUTINES                   */
/*                                         */
/* Group together the routines for reading */
/* the keymap and for processing keystrokes*/

void gnp_read_keymaps()

/* Read the keymaps XML file as pointed to */
/* by our config, and create the keymaps   */
/* memory structures used to guide input.  */

{
  xmlDocPtr  document;
  xmlNodePtr node,rootnode,innerstates;
  xmlNsPtr   namespace;
  guint      countstates=0;

  document=xmlParseFile("/home/conrad/Projects/gnuPOS/samplekeymap.xml");
  if(document==NULL)
  {
    printf("ERROR: Cannot read keymap file. ABORTING.\n");
    exit(-1);
  }

  node=xmlDocGetRootElement(document);
  if(node==NULL)
  {
    printf("ERROR: keymap file is empty. ABORTING.\n");
    exit(-1);
  }

  namespace=xmlSearchNsByHref(document,node,(const xmlChar *) "http://www.gnucash.org/gnuPOS/1.0");
  if(namespace==NULL)
  {
    printf("ERROR: keymap file of wrong type. ABORTING.\n");
    exit(-1);
  }
  if (xmlStrcmp(node->name, (const xmlChar *) "GnuPOS"))
  {
    printf("ERROR: GnuPOS data not found in file. ABORTING.\n");
    exit(-1);
  }

  rootnode=node;
  node=node->xmlChildrenNode;

  /* The tree is parsed in two passes. In the first pass */
  /* we parse all the definitions, but only count the    */
  /* number of states. After that pass is completed, we  */
  /* can grab enough memory for the state array, and we  */
  /* can parse the states confident that all definitions */
  /* have already been processed.                        */

  while(node!=NULL)
  {
    if(node&&xmlIsBlankNode(node))
    {
      /* printf("Blank node - name: %s, namespace %s\n",node->name, node->ns); */
      node=node->next;
    }
    else
      if((!xmlStrcmp(node->name,(const xmlChar *)"definitions"))&&(node->ns== namespace))
      {
        printf("definitions node\n");
        /*  parse definitions here  */

        node=node->next;
      }
      else
        if((!xmlStrcmp(node->name,(const xmlChar *)"states")) &&
            (node->ns == namespace))
        {
          /* Don't parse state tree here, just count them */
          innerstates=node->xmlChildrenNode;
          while(innerstates!=NULL)
          {
            if((!xmlStrcmp(innerstates->name,(const xmlChar *)"setstate"))&&
              (innerstates->ns==namespace))
            {
              countstates++;
              innerstates=innerstates->next;
            }
          }
          node=node->next;
        }
        else
        {
          printf("ERROR: Unknown node name %s\n",node->name);
          printf("   %s ignored.\n",node->name);
          node=node->next;
        }
  }

  /*  grab memory for states array  */
  printf("sizeof: %d\ncount: %d\n",sizeof(state_instance_struct),countstates);
  
  all_states.states=(state_instance_struct *)g_malloc(sizeof(state_instance_struct)*countstates);

  countstates=0;
  node=rootnode;
  while(node!=NULL)
  {
    if((!xmlStrcmp(node->name,(const xmlChar *)"states")) &&
        (node->ns == namespace))
    {
      /* This time, we parse the state tree */
      innerstates=node->xmlChildrenNode;
      while(innerstates!=NULL)
      {
        if((!xmlStrcmp(innerstates->name,(const xmlChar *)"setstate"))&&
          (innerstates->ns==namespace))
        {
          if(gnp_parse_state(innerstates,countstates))
            countstates++;
          innerstates=innerstates->next;
        }
      }
      node=node->next;
    }
    else
      node=node->next;
  }
}

gboolean gnp_parse_state(xmlNodePtr node, guint state)

/*  Do all the muck associated with building */
/* a complete state array entry for a state. */
/* node is the parent node of this state.    */
/* state is the array entry counter for the  */
/*     state array.                          */

{
  xmlNodePtr  np;

  /*  read name of state (its a property of node) */
  /*  and insert that into the state name lookup  */
  /*  table unless it is there already. If it is  */
  /*  a duplicate, print an error and return false*/
  /*  otherwise proceeed....                      */
  
  np=node->xmlChildrenNode;

  while(np!=NULL)
  {
    if(!xmlStrcmp(np->name,(const xmlChar *)"input"))
    {
      /* process input definition */
    }
    if(!xmlStrcmp(np->name,(const xmlChar *)"helptext"))
    {
      /* process helptext definition */
    }
  }

  return TRUE;
}

void gnp_process_keystroke(guint keyval)

/*   Take the keystroke returned by the    */
/*  callback, and perform all necessary    */
/*  processing to display stuff, etc.      */

{


}
